
# Tools

SVNLOOK=/usr/bin/svnlook
STYLECHECKER=/opt/bin/style_checker
PYTHON=/usr/bin/python
CEMAIL=/usr/lib/subversion/hook-scripts/commit-email.pl
BUILDBOT=/home/svn/scripts/svn_buildbot.py

TRACENV=/home/trac/rootenv
TRACURL=http://localhost:8000

# Path needed by some tools (style_checker)

PATH=/usr/bin:/bin
export PATH

# Style Check all files for a specific transaction

function check_style() {
   REPOS="$1"
   TXN="$2"
   CHECK_ARGS="$3"

   mkdir /tmp/$TXN

   # skip directories and deleted files

   $SVNLOOK changed -t "$TXN" "$REPOS" \
       | grep -v '/$' \
       | grep -v '^D' \
       > /tmp/$TXN/svn_changed

   styleerror=0

   for file in `awk '{print $2}' /tmp/$TXN/svn_changed`; do
      prop=`$SVNLOOK propget -t "$TXN" "$REPOS" svn:mime-type $file`
      # check only if a "text/*" or null mime-type
      if test -z "$prop" -o `echo $prop | grep text/ | wc -l` != 0; then
         tfile=/tmp/${TXN}/`basename $file`
         $SVNLOOK cat -t "$TXN" "$REPOS" $file > $tfile
         $STYLECHECKER $CHECK_ARGS $tfile
         if [ $? != 0 ]; then
            styleerror=1;
            echo "Style error for file" $file 1>&2
         fi;
      fi;
   done

   rm -fr /tmp/$TXN
   return $styleerror;
}

# Check log not empty

function log_not_empty() {
   REPOS="$1"
   TXN="$2"

   RES=`$SVNLOOK log -t "$TXN" "$REPOS" | grep "[a-zA-Z0-9]"`

   if [ "$RES" = "" ]; then
      echo "Won't commit with an empty log message." 1>&2
      $SVNLOOK changed -t "$TXN" $REPOS 1>&2
      return 1;
   fi

   return 0;
}

# Post commit action to log message in Trac

function trac_post_commit_record_log() {
   REPOS="$1"
   REV="$2"
   MODULE="$3"

   LOG=`$SVNLOOK log -r $REV $REPOS`
   AUTHOR=`$SVNLOOK author -r $REV $REPOS`

   $PYTHON /home/svn/scripts/trac-post-commit-hook \
      -p "${TRACENV}/$MODULE"  \
      -r "$REV"       \
      -u "$AUTHOR"    \
      -m "$LOG"       \
      -s "${TRACURL}/$MODULE" 2> /tmp/post_commit_err_${REV}
}

# Pre commit action to log message in Trac

function trac_pre_commit_check_log() {
   REPOS="$1"
   TXN="$2"
   MODULE="$3"
   
   LOG=`$SVNLOOK log -t $TXN $REPOS`

   $PYTHON /home/svn/scripts/trac-pre-commit-hook \
	"${TRACENV}/$MODULE" "$LOG" || return 1

   return 0;
}

# Post commit send mail

function send_mail_post_commit() {
    REPOS="$1"
    REV="$2"
    SUBJECT=$3

    $CEMAIL -s "$SUBJECT" -h cln46mr "$REPOS" "$REV" $4 $5 $6 $7 $8 $9
}

# Buildbot Post commit

function buildbot_post_commit () {
    REPOS="$1"
    REV="$2"
    PORT=9989

    $BUILDBOT --repository "$REPOS" --revision "$REV" --bbserver localhost --bbport $PORT
}
