#!/usr/bin/env python2.3

# this requires python >=2.3 for the 'sets' module.

# The sets.py from python-2.3 appears to work fine under python2.2 . To
# install this script on a host with only python2.2, copy
# /usr/lib/python2.3/sets.py from a newer python into somewhere on your
# PYTHONPATH, then edit the #! line above to invoke python2.2

# python2.1 is right out

# If you run this program as part of your SVN post-commit hooks, it will
# deliver Change notices to a buildmaster that is running a PBChangeSource
# instance.

# edit your svn-repository/hooks/post-commit file, and add lines that look
# like this:

'''
# set up PYTHONPATH to contain Twisted/buildbot perhaps, if not already
# installed site-wide
. ~/.environment

/path/to/svn_buildbot.py --repository "$REPOS" --revision "$REV" --bbserver localhost --bbport 9989
'''

import commands, sys, os
import re
import sets

# We have hackish "-d" handling here rather than in the Options
# subclass below because a common error will be to not have twisted in
# PYTHONPATH; we want to be able to print that error to the log if
# debug mode is on, so we set it up before the imports.

DEBUG = None

if '-d' in sys.argv:
    i = sys.argv.index('-d')
    DEBUG = sys.argv[i+1]
    del sys.argv[i]
    del sys.argv[i]

if DEBUG:
    f = open(DEBUG, 'a')
    sys.stderr = f
    sys.stdout = f

from twisted.internet import reactor
from twisted.python import usage
from twisted.spread import pb
from twisted.cred import credentials

class Options(usage.Options):
    optParameters = [
        ['repository', 'r', None,
         "The repository that was changed."],
        ['revision', 'v', None,
         "The revision that we want to examine (default: latest)"],
        ['bbserver', 's', 'localhost',
         "The hostname of the server that buildbot is running on"],
        ['bbport', 'p', 8007,
         "The port that buildbot is listening on"]
    ]
	
    def postOptions(self):
        if self['repository'] is None:
            raise usage.error("You must pass --repository")


def split_by_branch(changes):
    """Return a dictionary mapping branches to change sets."""

    re_branch = re.compile(r'^((?P<branch>branches/[^/]*)|(?P<trunk>trunk))/')

    branches = {}
    for f in changes:
        match = re_branch.search(f)
        if match:
            branch = match.group('branch') or 'trunk'
            if not branches.get(branch):
                branches[branch] = []
            branches[branch].append(f)
    return branches


def main(opts):
    repo = opts['repository']
    print "Repo:", repo
    rev_arg = ''
    if opts['revision']:
        rev_arg = '-r %s' % (opts['revision'],)
    changed = commands.getoutput('svnlook changed %s "%s"' % (rev_arg, repo)
                                 ).split('\n')
    changed = [x[1:].strip() for x in changed]
    message = commands.getoutput('svnlook log %s "%s"' % (rev_arg, repo))
    who = commands.getoutput('svnlook author %s "%s"' % (rev_arg, repo))
    revision = opts.get('revision')
    if revision is not None:
        revision = int(revision)

    # construct one change notification per modified branch
    branches = split_by_branch(changed)

    pbcf = pb.PBClientFactory()
    reactor.connectTCP(opts['bbserver'], int(opts['bbport']),
                       pbcf)

    def add_branch(o, branch, files):

        def gotPersp(persp):
            change = {}
            change['who'] = who
            change['files'] = files
            if branch: change['branch'] = branch
            change['comments'] = "%s %s" % (repo, message)
            change['revision'] = revision
            for k, v in change.items():
                print k, repr(v)
            return persp.callRemote('addChange', change)

        return o.addCallback(gotPersp)

    def quit(*why):
        print "quitting! because", why
        reactor.stop()


    o = pbcf.login(credentials.UsernamePassword('change', 'changepw'))
    for b in branches:
        o = add_branch(o, b, branches[b])
    o.addCallback(quit, "SUCCESS").addErrback(quit, "FAILURE")
    # timeout of 60 seconds
    reactor.callLater(60, quit, "TIMEOUT")

    reactor.run()

if __name__ == '__main__':
    opts = Options()
    try:
        opts.parseOptions()
    except usage.error, ue:
        print opts
        print "%s: %s" % (sys.argv[0], ue)
        sys.exit()
    main(opts)
